@@ -197,17 +197,18 @@ def modeling_model(model, med_type):
 
 def grad_lmd_to_vd(glam, gmu, grho, lam, mu, rho):
     """
-    grad_lmr_to_vd [summary]
+    grad_lmr_to_vd switch the gradient.
 
-    [extended_summary]
+    This function witch the gradient from (lambda, mu, rho)
+    to (vp, vs, rho).
 
     Args:
-        glam ([type]): [description]
-        gmu ([type]): [description]
-        grho ([type]): [description]
-        lam ([type]): [description]
-        mu ([type]): [description]
-        rho ([type]): [description]
+        glam (ndarray): Gradient w.r.t. lambda
+        gmu (ndarray): Gradient w.r.t. mu
+        grho (ndarray): Gradient w.r.t. density
+        lam (ndarray): Gradient w.r.t. lambda
+        mu (ndarray): Gradient w.r.t. mu
+        rho (ndarray): Gradient w.r.t. density
     
     Refrences:
          1. Hu et al, 2021, Direct updating of rock-physics properties using elastice full-waveform inversion
@@ -215,25 +216,23 @@ def grad_lmd_to_vd(glam, gmu, grho, lam, mu, rho):
     """
     vp = np.sqrt((lam + 2 * mu) / rho)
     vs = np.sqrt(mu / rho)
+    vs2 = vs ** 2
+    vpvs = vp ** 2 - 2 * vs ** 2
     
     glam_vp = glam * 2 * vp * rho
-    gmu_vp = gmu * rho * vp
-    grho_vp = grho * (- 2 * (lam + 2*mu)/vp**3)
+    gmu_vp = gmu * 0
+    grho_vp = grho * 0
     gvp = glam_vp + gmu_vp + grho_vp  
 
-    glam_vs = glam * 0
+    glam_vs = glam * (-4 * rho * vs)
     gmu_vs = gmu * 2 * vs * rho
     
-    # To not get ZeroDivisionError in acoustic case
-    if np.all(vs==0):
-        grho_vs  = 0
-    else:
-        grho_vs = grho * (-2*mu/(vs ** 3))
+    grho_vs = grho * 0
 
     gvs = glam_vs + gmu_vs + grho_vs  # gvs
 
-    glam_rho = glam * vp ** 2
-    gmu_rho = gmu * 0.5 * vp ** 2
+    glam_rho = glam * vpvs
+    gmu_rho = gmu * vs2
     grho_rho = grho
     grho = glam_rho + gmu_rho + grho_rho
 
@@ -258,6 +257,7 @@ def adj_grad_lmd_to_vd(gvp, gvs, grho, lam, mu, rho):
          1. Hu et al, 2021, Direct updating of rock-physics properties using elastice full-waveform inversion
          2. Zhou and Lumely, 2021, Central-difference time-lapse 4D seismic full-waveform inversion
     """
+    print(" This function is not stable.")
     vp = np.sqrt((lam + 2 * mu) / rho)
     vs = np.sqrt(mu / rho)
     
@@ -276,138 +276,7 @@ def adj_grad_lmd_to_vd(gvp, gvs, grho, lam, mu, rho):
     grho_rho = grho
     grho = grho_vp + grho_vs + grho_rho
 
-    return glam.astype(np.float32), gmu.astype(np.float32), grho.astype(np.float32)
-
-
-def grad_vd2pcs(gvp0, gvs0, grho0, cc, phi, sw):
-    """
-    grad_vd2pcs [summary]
-
-    [extended_summary]
-
-    Args:
-        gvp ([type]): [description]
-        gvs ([type]): [description]
-        grho ([type]): [description]
-        cc ([type]): [description]
-        rho_c ([type]): [description]
-        rho_q ([type]): [description]
-        phi ([type]): [description]
-        rho_w ([type]): [description]
-        rho_g ([type]): [description]
-        rho_f ([type]): [description]
-
-    Returns:
-        [type]: [description]
-
-    Refrences:
-         1. Hu et al, 2021, Direct updating of rock-physics properties using elastice full-waveform inversion
-         2. Zhou and Lumely, 2021, Central-difference time-lapse 4D seismic full-waveform inversion
-    """
-    rho_q = 2.65
-    rho_c = 2.55
-    rho_w = 1.0
-    rho_g = 0.1
-
-    a1=5.5
-    a2=6.9
-    a3=2.2
-    b1=3.4
-    b2=4.7
-    b3=1.8
-     
-    rho_f = rp.Density().fluid(rho_g, rho_w, sw)
-
-    gvp = np.copy(gvp0)
-    gvs = np.copy(gvs0)
-    grho = np.copy(grho0)
-
-    rho_m = rp.Density().matrix(rho_c, cc, rho_q)
-
-    gvp_phi = gvp * (- a2 * 1)
-    gvs_phi = gvs * (- b2 * 1)
-    grho_phi = grho * (- rho_m + rho_f)
-    gphi = gvp_phi + gvs_phi + grho_phi  
-
-    gvp_cc = gvp * (- a3 * 1)
-    gvs_cc = gvs * (- b3 * 1)
-    grho_cc = grho * (1 - phi) * (rho_c - rho_q)
-    gcc = gvp_cc + gvs_cc + grho_cc  
-
-    gvp_s = gvp * 0
-    gvs_s = gvs * 0
-    grho_s = grho * phi * (rho_w - rho_g)
-    gs = gvp_s + gvs_s + grho_s
-
-    return gphi, gcc, gs
-
-def grad_pcs2vd(gphi0, gcc0, gsw0, cc, phi, sw):    
-    """
-    grad_pcs2vd [summary]
-
-    [extended_summary]
-
-    Args:
-        gvp ([type]): [description]
-        gvs ([type]): [description]
-        grho ([type]): [description]
-        cc ([type]): [description]
-        rho_c ([type]): [description]
-        rho_q ([type]): [description]
-        phi ([type]): [description]
-        rho_w ([type]): [description]
-        rho_g ([type]): [description]
-        rho_f ([type]): [description]
-
-    Returns:
-        [type]: [description]
-
-    Refrences:
-         1. Hu et al, 2021, Direct updating of rock-physics properties using elastice full-waveform inversion
-         2. Zhou and Lumely, 2021, Central-difference time-lapse 4D seismic full-waveform inversion
-    """
-    a1=5.5
-    a2=6.9
-    a3=2.2
-    b1=3.4
-    b2=4.7
-    b3=1.8
-        
-    rho_q = 2.65
-    rho_c = 2.55
-    rho_w = 1.0
-    rho_g = 0.1
-
-    rho_f = rp.Density().fluid(rho_g, rho_w, sw)
-
-    gphi = np.copy(gphi0)
-    gcc = np.copy(gcc0)
-    gsw = np.copy(gsw0)
-
-    rho_m = rp.Density().matrix(rho_c, cc, rho_q)
-
-    gphi_vp = gphi * (1 / (- a2 * 1))
-    gcc_vp = gcc * (1 / (- a2 * 1))
-    gsw_vp = gsw * 0
-    gvp = gphi_vp + gcc_vp + gsw_vp  
-
-    gphi_vs = gphi * (1 / (- b2 * 1))
-    gcc_vs = gcc * (1 / (- b3 * 1))
-    gsw_vs = gsw * 0
-    gvs = gphi_vs + gcc_vs + gsw_vs  
-
-    gphi_rho = gphi * 1 / (rho_f - rho_m)
-    gcc_rho = gcc * 1 / (rho_w - rho_q)
-    
-    
-    gsw_rho0 = ((1 / rho_w - rho_g) +
-                ((rho_f - rho_g)/(sw * (rho_w - rho_g) ** 2)) +
-                ((rho_f - rho_w)/((1 - sw) * (rho_w - rho_g) ** 2))) /phi
-    
-    gsw_rho = gsw * gsw_rho0
-    grho = gphi_rho + gcc_rho + gsw_rho
-
-    return gvp, gvs, grho
+    return glam.astype(np.float32), gmu.astype(np.float32), grho.astype(np.float32) 
 
 
 class recorder:
@@ -628,8 +497,6 @@ def pml_counstruction(TNz, TNx, dh,
     PML construction generate two matrices for x- and z-directions with the
     size of velocity model plus number of pml samples in each direction.
 
-    Extended Summary
-    ----------------
     dx_pml and dz_pml are obtained based on Gao et al., 2017, Comparison of
     artiﬁcial absorbing boundaries for acoustic wave equation modelling.
 
@@ -702,8 +569,6 @@ def pml_delta_calculation(dh, n_pml=10, pml_r=1e-5):
         This function generates delta vector for PML construction function which put this vector
         around the model matrices.
 
-        Extended Summary
-        ----------------
         dx_pml and dz_pml are obtained based on Gao et al., 2017, Comparison of
         artiﬁcial absorbing boundaries for acoustic wave equation modelling.
 
@@ -757,7 +622,21 @@ def vel_dict2vec(m0):
     return m
 
 
-def vec2vel_dict(m0, nz, nx):        
+def vec2vel_dict(m0, nz, nx):  
+    """
+    vec2vel_dict converts a vector of DV to dictionary
+
+    This function converts a vector of DV to dictionary which is 
+    used during the inversion.
+
+    Args:
+        m0 (1-d ndarray): a vector containg the whole parameters of the model
+        nz ([type]): Number of samples of the model in z-direction
+        nx ([type]): Number of samples of the model in x-direction
+
+    Returns:
+        m (dictionary): A dictionary ccontaining 'vp', 'vs', 'rho'.
+    """
     m = {
         'vp': m0[:nz * nx].reshape(nz, nx),
         'vs': m0[nz * nx:2*nz * nx].reshape(nz, nx),
@@ -768,6 +647,18 @@ def vec2vel_dict(m0, nz, nx):
 
 
 def pcs_dict2vec(m0):
+    """
+    pcs_dict2vec converts a dictionary of PCS to a vector
+
+    This function converts a dictionary of PCS to vector which is 
+    used during the inversion.
+
+    Args:
+        m0 (dictionary): A dictionary ccontaining 'phi', 'cc', 'sw'.
+
+    Returns:
+        m (dictionary): A vector containg the whole parameters of the model.
+    """
     nz, nx = m0[[*m0][0]].shape
     m = np.zeros((3 * nz * nx))
     
@@ -777,7 +668,21 @@ def pcs_dict2vec(m0):
     return m
 
 
-def vec2pcs_dict(m0, nz, nx):        
+def vec2pcs_dict(m0, nz, nx):  
+    """
+    vec2pcs_dict converts a vector of PCS to dictionary
+
+    This function converts a vector of PCS to dictionary which is 
+    used during the inversion.
+
+    Args:
+        m0 (1-d ndarray): a vector containg the whole parameters of the model
+        nz ([type]): Number of samples of the model in z-direction
+        nx ([type]): Number of samples of the model in x-direction
+
+    Returns:
+        m (dictionary): A dictionary ccontaining 'phi', 'cc', 'sw'.
+    """      
     m = {
         'phi': m0[:nz * nx].reshape(nz, nx),
         'cc': m0[nz * nx:2*nz * nx].reshape(nz, nx),
@@ -795,14 +700,31 @@ def svd_reconstruction(m, begining_component, num_components):
 
 
 def cost_preparation(dpre, dobs,
-                     fn, freq=False, order=None, axis=None,
+                     fn, freq=None, order=None, axis=None,
                      params_oh=None):
+    """
+    cost_preparation prepare the data for calculating the cost function
+
+    This function prepare the data for calculating the cost function.
+    This preparation is based on multi-scale inversion strategy (Bunks et al., 1995).
+
+    Args:
+        dpre ([type]): Predicted data
+        dobs ([type]): Observed data
+        fn (float): Nyquist frequency
+        freq (float, optional): Desire frequency for filtering. Defaults to None.
+        order (int, optional): Order of the filter. Defaults to None.
+        axis (int, optional): Axis of the filter. Defaults to None.
+        params_oh ([type], optional): Parameter to prepare the data for different offsets. Defaults to None.
+
+    Returns:
+        [type]: [description]
+    """
 
     x_pre = copy.deepcopy(dpre)
     x_obs = copy.deepcopy(dobs)
 
     if freq:
-        # highcut = freq / fn
         x_obs = lowpass(x_obs, freq, fn, order=order, axis=axis)
         x_pre = lowpass(x_pre, freq, fn, order=order, axis=axis)
 
@@ -915,16 +837,40 @@ def source_weighting(d_pre, d_obs, ns, nr):
 
 
 def cost_seismic(d_pre0, d_obs0, fun,
-         fn=None, freq=False, order=None, axis=None,
+         fn=None, freq=None, order=None, axis=None,
          sourc_weight=False, ns=None, nr=None,
          params_oh=None):
+    """
+    cost_seismic calculates the cost between estimated and observed data.
+
+    This function calculates the cost between estimated and observed data by applying desired filters 
+    and returns the cost and the adjoint of the residual.
+
+    Args:
+        d_pre0 (dict): Estimated data
+        d_obs0 (dict): Observed data
+        fun (function): The function to calculate the cost. This could be "CF = tools.CostFunction('l2')"
+        fn (float, optional): Nyquist frequency. Defaults to None.
+        freq (float, optional): Desired frequency to implement the lowpass filter. Defaults to None.
+        order ([type], optional): [description]. Defaults to None.
+        axis ([type], optional): [description]. Defaults to None.
+        sourc_weight (bool, optional): [description]. Defaults to False.
+        ns (int, optional): Number of the sources. Defaults to None.
+        nr (int, optional): Number of the receivers. Defaults to None.
+        params_oh ([type], optional): [description]. Defaults to None.
+
+    Returns:
+        rms (float): The cost
+        adj_src: Adjoint source to propagate through the model in adjoint wave equation
+    """
+
 
     d_pre = copy.deepcopy(d_pre0)
     d_obs = copy.deepcopy(d_obs0)
 
     dpre = np.array(list(d_pre.values()))
     dobs = np.array(list(d_obs.values()))
-
+    # TODO: Treat as dict
     x_pre_cost, x_obs_cost = cost_preparation(dpre, dobs,
                                               fn, freq=freq, order=order, axis=axis,
                                               params_oh=params_oh)
@@ -949,6 +895,10 @@ def cost_seismic(d_pre0, d_obs0, fun,
 
 
 class CostFunction:
+    """
+     This class provide different cost functions. 
+
+    """
     def __init__(self, cost_function_type="l2"):
         self.cost_function_method = "self." + cost_function_type
 
@@ -970,6 +920,7 @@ class CostFunction:
         
     def l1(self, dest, dobs):
         res = np.float32(dest - dobs)
+        #TODO: adj_src is not right
         rms = np.sum(np.abs(res))
         adj_src = res  # np.ones(res.shape, np.float32)
 
@@ -989,8 +940,6 @@ class CostFunction:
         for param in dest:
             res[param] = np.float32(dest[param] - dobs[param])
             rms += 0.5 * (res[param].reshape(-1).T @ res[param].reshape(-1))
-
-        # adj_src = res
         
         if type(dest0).__name__ == 'ndarray':
             adj_src = np.array(list(res.values()))
@@ -1000,7 +949,7 @@ class CostFunction:
         return rms, adj_src
 
     def l2_intensity(self, dest, dobs):
-        # res = [dest[i] - dobs[i] for i in range(len(dest))]
+
         res = dest**2 - dobs**2
         rms = 0.25 * (res.reshape(-1).T @ res.reshape(-1))
 
