@@ -15,7 +15,7 @@ import PyFWI.acquisition as acq
 
 
 class wave_preparation():
-    def __init__(self, inpa, src, rec_loc, model_size, n_well_rec=0, chpr=10, components=4):
+    def __init__(self, inpa, src, rec_loc, model_size, n_well_rec=0, chpr=10, components=0):
         '''
         A class to prepare the variable and basic functions for wave propagation.
         
@@ -628,14 +628,14 @@ class wave_propagator(wave_preparation):
         for t in range(1, self.nt):
 
             if W is None:
-                src_kt_x, src_kt_z = np.float32(self.src(t))
+                src_kv_x, src_kv_z, src_kt_x, src_kt_z, src_kt_xz = np.float32(self.src(t))
                 
                 self.prg.injSrc(self.queue, (self.tnz, self.tnx), None,
                             self.vx_b, self.vz_b,
                             self.taux_b, self.tauz_b, self.tauxz_b,
                             self.rho_b,
                             self.srcx[s], self.srcz[s],
-                            src_kt_x, src_kt_z)
+                            src_kv_x, src_kv_z, src_kt_x, src_kt_z, src_kt_xz)
             else:
                 if t in self.chp:
                         vx = W['vx'][:, :, s, chp_count_adjoint].astype(np.float32, order='C')
@@ -890,13 +890,15 @@ class wave_propagator(wave_preparation):
         
         self.pml_preparation(model['vp'].max())
         self.elastic_buffers(model)
-        seismo = self.forward_propagator(model, W, grad)    
-        return seismo
+        seismo = self.forward_propagator(model, W, grad) 
+        data = acq.seismic_section(seismo, self.components)
+        return data
     
     def gradient(self, res, show=False, Lam=None, grad=None, parameterization='dv'):
         self.backward_show = show
         self.adjoint_buffer_preparing()
         
+        res = acq.prepare_residual(res, 1)
         if show:
             self.initial_wavefield_plot({'vp':self.vp}, plot_type="Backward")
         
@@ -921,98 +923,3 @@ class wave_propagator(wave_preparation):
         
         return final_grad
         
-        
-if __name__ == "__main__":
-    import PyFWI.model_dataset as md
-    import PyFWI.acquisition as acq
-    import PyFWI.seiplot as splt
-    
-    GRADIENT = True  #  False
-    INVERSION = False
-    
-    model_gen = md.ModelGenerator('yang') # louboutin') # 
-    
-    model = model_gen()
-    # model_gen.show(['vs'])
-    model_shape = model[[*model][0]].shape
-    
-    inpa = {}
-    # Number of pml layers
-    inpa['npml'] = 5
-    inpa['pmlR'] = 1e-5
-    inpa['pml_dir'] = 2
-    inpa['device'] = 2
-    inpa['energy_balancing'] = False
-    chpr = 100
-    sdo = 4
-    fdom = 25
-    inpa['fn'] = 125
-    vp = model['vp']
-    D = seis_process.derivatives(order=sdo)
-    dh = vp.min()/(D.dh_n * inpa['fn'])
-    dh = 2.
-    inpa['dh'] = dh
-    
-    dt = D.dt_computation(vp.max(), inpa['dh'])
-    inpa['dt'] = dt
-    
-    print(f'{dh = } ........... {dt = }')
-    inpa['t'] = 0.36
-    
-    offsetx = inpa['dh'] * model_shape[1]
-    depth = inpa['dh'] * model_shape[0]
-
-    inpa['rec_dis'] = 2.  # inpa['dh']
-    ns = 2
-    inpa['acq_type'] = 0
-
-    src_loc, rec_loc, n_surface_rec, n_well_rec = acq.AcqParameters(ns, inpa['rec_dis'], offsetx, depth, inpa['dh'], sdo, inpa['acq_type'])
-    
-    src = acq.Source(src_loc, inpa['dh'], inpa['dt'])
-    src.Ricker(fdom)
-    
-    W = wave_propagator(inpa, src, rec_loc, model_shape, n_well_rec, chpr=0)
-    d_obs = W.forward_modeling(model, False)
-    
-    m0 = model_gen(vintage=1, smoothing=True)
-    
-    if GRADIENT:
-        Lam = wave_propagator(inpa, src, rec_loc, model_shape, n_well_rec, chpr=chpr)
-        d_est = Lam.forward_modeling(m0, show=False)
-
-        CF = tools.CostFunction('l2')
-        rms, res = CF(d_est, d_obs)
-
-        print(rms)
-    
-        # fig = plt.figure()
-        # ax = fig.add_subplot(1, 3, 1)
-        # seiplt.seismic_section(ax, d_obs['taux'], vmin=d_obs['taux'].min() / 5, vmax=d_obs['taux'].max() / 5)
-        # ax = fig.add_subplot(1, 3, 2)
-        # seiplt.seismic_section(ax, d_est['taux'])
-        # ax = fig.add_subplot(1, 3, 3)
-        # seiplt.seismic_section(ax, res['taux'])
-        # plt.show()
-        
-        grad = Lam.gradient(res, show=False)
-        
-        splt.earth_model(grad, cmap='jet')
-        plt.show()
-        a=1
-    
-    elif INVERSION:
-        import PyFWI.optimization as fwi
-        
-        FWI = fwi.FWI(d_obs, inpa, src, rec_loc, model_shape, n_well_rec, chpr=chpr, components=4)
-        inverted_model, rms = FWI(m0, method=1, iter=3, freqs=[25])
-        
-        
-        splt.earth_model(inverted_model, cmap='jet')
-        plt.show(block=False)
-        
-        plt.figure()
-        plt.plot(rms/max(rms))
-        plt.show()
-        
-
-    
\ No newline at end of file
